LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;

entity cont8_as_reset_up_tb is
end cont8_as_reset_up_tb;

architecture testbench of cont8_as_reset_up_tb is

  component cont8_as_reset_up is
    port (clock : in  std_logic;
          reset : in  std_logic;
          up    : in  std_logic;
          Z     : out std_logic_vector(7 downto 0)
         );
  end component;

  signal Z      : std_logic_vector (7 downto 0);
  signal clock     : std_logic;
  signal reset     : std_logic;
  signal up     : std_logic;
  signal ris, flag_rst, flag_up : std_logic := '0';
  signal t : std_logic_vector(7 downto 0); -- dovrebbe assumere il valore precedente assunto da Z
begin
  uut : cont8_as_reset_up port map (clock, reset, up, Z);

  -- finche' ris vale 0 tutto dovrebbe andare bene.
  ris <= '0' when (((flag_rst = '1' and Z = "00000000") or
                   ((up xor flag_up) = '1' and Z = t + "00000001") or
                   ((up xor flag_up) = '0' and Z = t - "00000001"))
                   and (not is_X(Z))
                  )
         else '1';

  flag_rst_p : process(reset, clock)
    -- flag_rst viene usato per segnalare un'azione di reset.
    -- flag_rst viene posto a 1 da reset,
    -- ma viene rimesso a 0 dal clock successivo,
    -- se reset  tornato a 0
  begin
    if reset = '1' then
      flag_rst <= '1';
    elsif clock'event and clock = '1' then
      flag_rst <= '0';
    end if;
  end process;


  flag_up_p : process(up, clock)
    -- flag_up viene usato per segnalare un cambiamento di up.
    -- flag_up viene posto a 1 da un evento su up,
    -- ma viene rimesso a 0 dal clock successivo,
    -- o da un altro cambiamento di up
  begin
    if up'event then
      flag_up <= not flag_up;
    elsif clock'event and clock = '1' then
      flag_up <= '0';
    end if;
  end process;

  reset_p : process
  begin
    reset <= '1';
    wait for 1 ns;
    reset <= '0';
    wait for 23 ns;
    reset <= '1';
    wait for 1 ns;
    reset <= '0';
    wait for 10 ns;
    reset <= '1';
    wait for 1 ns;
    reset <= '0';
    wait for 100 ns;
    reset <= '1';
    wait for 1 ns;
    reset <= '0';
    wait for 56 ns;
    reset <= '1';
    wait for 1 ns;
  end process;

  up_p : process
  begin
    up <= '1';
    wait for 103 ns;
    up <= '0';
    wait for 59 ns;
    up <= '1';
    wait for 32 ns;
    up <= '0';
    wait for 2 ns;
    up <= '1';
    wait for 20 ns;
    up <= '0';
    wait for 208 ns;
  end process;

  clock_p : process
  begin
    clock <= '0';
    wait for 5 ns;
    t <= Z;
    clock <= '1';
    wait for 5 ns;
  end process;

end testbench;







